#include <cvirte.h>     
#include <userint.h>
#include "rsspecan_3gpp_example.h"
#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "rsspecan.h"


/* Global variables */
static      ViInt32 panelHandle      = VI_NULL;
volatile    ViInt32 rsspecan_status  = VI_SUCCESS; 
volatile    ViInt32 gRun             = 0;
volatile    ViInt32 gSweep           = 0;
static      ViSession   instrSession = VI_NULL;
static      ViStatus    specan_status= VI_SUCCESS;;


/* Utility functions declarations */
void cleanup (ViBoolean deviceClear);
void InsertRow (ViInt32 row, ViString name, ViReal64 result, ViString units);
void EnableButtons (ViBoolean enable);


/* Useful macros */
#define CHECKERR(fcal) if ((rsspecan_status = fcal) < 0)\
                       {\
                            cleanup(VI_FALSE);\
                            return rsspecan_status;\
                       }

/*
 * Useful macros used in timerCallback function to ensure propper
 * bus-reset in case of an missing/faulty external trigger
 */
#define CHECKERRR(fcal) if ((rsspecan_status = fcal) < 0)\
                       {\
                            cleanup(VI_TRUE);\
                            return rsspecan_status;\
                       }
#define WINDOW  1
#define SWEEPS  0
#define TIMEOUT 15000 
#define STRING_LENGTH 512

/* Power condition status bits */
#define STAT_POW_OVERLOAD_SCR_A     0x1
#define STAT_POW_UNDERLOAD_SCR_A    0x2
#define STAT_POW_IF_OVERLOAD_SCR_A  0x4


/* Sync condition status bits */
#define STAT_SYNC_PILOT         0x10        /* Bit 5 */
#define STAT_SYNC_FRAME_SYNC    0x02        /* Bit 1 */


/*===========================================================================*/
/* Function: Main function
/* Purpose:  Application entry, displays the GUI
/*===========================================================================*/
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsspecan_3gpp_example.uir", PANEL)) < 0)
        return -1;
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            QuitUserInterface (0);
            break;
    }
    return 0;
}

int CVICALLBACK timerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    ViReal64    result      = 0;
    ViInt32     line        = 1;
    ViInt32     status_reg  = 0;
    
    switch (event)
    {
        case EVENT_TIMER_TICK:
            if (gRun)
            {
                gRun=0; /* ensure exclusive acess for single sweep */
            
                /*
                 * Send trigger (initiate measurement) and wait for *OPC. 
                 * If this functions fails (usually because
                 * of timeout) it is necessary to send
                 * Clear Device and *CLS to instrument
                 */
                CHECKERRR (rsspecan_Initiate (instrSession, WINDOW, TIMEOUT)); 
                
                /*
                 * After measurement is finished check the overload and underload
                 * conditions. Consult remote control manual for more information
                 * about meaning of each bit of the register.
                 */
                CHECKERRR (rsspecan_ConditionPowerRegister(instrSession, &status_reg));
                if (status_reg & STAT_POW_OVERLOAD_SCR_A)
                {
                    MessagePopup ("ERROR", "RF Input Overloaded (Screen A)!");
                    cleanup (VI_FALSE);
                    return -1;
                }
                
                else if (status_reg & STAT_POW_UNDERLOAD_SCR_A)
                {
                    MessagePopup ("ERROR", "RF Input Underloaded (Screen A)!");
                    cleanup (VI_FALSE);
                    return -2;
                }
                
                else if (status_reg & STAT_POW_IF_OVERLOAD_SCR_A)
                {
                    MessagePopup ("ERROR", "IF Overload!");
                    cleanup (VI_FALSE);
                    return -3;
                }
                else if (status_reg != 0) /* various error */
                {
                    MessagePopup ("ERROR", "Undefined Error!");
                    cleanup (VI_FALSE);
                    return -3;
                }
                                
                /*
                 * After measurement is finished check the synchronization
                 * conditions. Consult remote control manual for more information
                 * about meaning of each bit of the register.
                 */
                CHECKERRR (rsspecan_ConditionSyncRegister(instrSession, &status_reg));
                if (status_reg & STAT_SYNC_PILOT)
                {
                    MessagePopup ("ERROR", "Incorrect Pilot Symbol!");
                    cleanup (VI_FALSE);
                    return -4;
                }

                else if (status_reg & STAT_SYNC_FRAME_SYNC)
                {
                    MessagePopup ("ERROR", "Frame Synchronization Failed!");
                    cleanup (VI_FALSE);
                    return -5;
                }
                else if (status_reg != 0) /* various error */
                {
                    MessagePopup ("ERROR", "Undefined Error!");
                    cleanup (VI_FALSE);
                    return -3;
                }
                
                /*
                 * If and only if the synchronization is correct, query the
                 * results from the instrument.
                 */
                
                /* Total Power */
                CHECKERRR (rsspecan_Get3GPPBSResult(instrSession, RSSPECAN_VAL_WCDP_RES_PTOT , &result));
                InsertRow (line, "Total Power", result, "dBm");
                line++;
                
                /* Chip Rate Error */
                CHECKERRR (rsspecan_Get3GPPBSResult(instrSession, RSSPECAN_VAL_WCDP_RES_CERR, &result));
                InsertRow (line, "Chip Rate Error", result, "ppm");
                line++;

                /* IQ Offset */
                CHECKERRR (rsspecan_Get3GPPBSResult(instrSession, RSSPECAN_VAL_WCDP_RES_IQOF, &result));
                InsertRow (line, "IQ Offset", result, "%");
                line++;

                /* Error Vector Magnitude RMS */
                CHECKERRR (rsspecan_Get3GPPBSResult(instrSession, RSSPECAN_VAL_WCDP_RES_EVMR, &result));
                InsertRow (line, "Error Vector Magnitude RMS", result, "%");
                line++;

                /* IQ Imbalance */
                CHECKERRR (rsspecan_Get3GPPBSResult(instrSession, RSSPECAN_VAL_WCDP_RES_IQOF, &result));
                InsertRow (line, "IQ Imbalance", result, "%");
                line++;

                /* Peak Code Domain Error */
                CHECKERRR (rsspecan_Get3GPPBSResult(instrSession, RSSPECAN_VAL_WCDP_RES_PCD, &result));
                InsertRow (line, "Peak Code Domain Error", result, "dB");
                line++;
                
                
                if (gSweep)     /* single or continous sweep? */
                {
                    gRun = 1;   /* continue */
                }
                else
                {
                    gRun = 0;   /* finish measurement */
                    EnableButtons (1); 
                }
            }
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stops the continuous measurement, in other cases has no sense
/*===========================================================================*/
int CVICALLBACK stopButtonCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            
            /* stop measuring  */
            gRun = 0;
            
            /* close session */
            cleanup(VI_FALSE);
            
            /* enable GUI forms */
            EnableButtons (1);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int CVICALLBACK startButtonCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32     display;
    ViChar      instrDescr[128];
    ViReal64    frequency;
    ViReal64    level;
    ViInt32     trigger_mode;
    ViInt32     id = 0;
    ViInt32     reset = VI_TRUE;

    switch (event)
    {
        case EVENT_COMMIT:
            
            /* disable GUI form */
            SetWaitCursor(1);
            EnableButtons (0);
            
            /* read config values form GUI forms */
            GetCtrlVal (panelHandle, PANEL_RESOURCE_STRING, instrDescr);
            GetCtrlVal (panelHandle, PANEL_FREQ_NUMERIC,    &frequency);
            GetCtrlVal (panelHandle, PANEL_LEVEL_NUMERIC,   &level);
            GetCtrlVal (panelHandle, PANEL_TRIGGER_RING,    &trigger_mode);
            GetCtrlVal (panelHandle, PANEL_SWEEP_RING,      &gSweep);
            GetCtrlVal (panelHandle, PANEL_DISPLAY_CHECKBOX,&display);
            GetCtrlVal (panelHandle, PANEL_RESET_CHECKBOX,  &reset);
            
            /* Open the session */
            CHECKERR (rsspecan_init (instrDescr, id, reset, &instrSession));
            
            /* Enable / Disable the display during update during remote control */
            CHECKERR( rsspecan_ConfigureDisplayUpdate (instrSession, display));
            
            /* Switch to 3G FDD BS Instrument Mode */
            CHECKERR( rsspecan_Set3GPPBSMode(instrSession));
            
            /* Select Code Domain Power Measurement */
            CHECKERR (rsspecan_Set3GPPBSMeasurement(instrSession, RSSPECAN_VAL_3GPPBS_MEAS_WCDP));
            
            /* Configure center frequency to desired value */
            CHECKERR (rsspecan_ConfigureFrequencyCenter (instrSession, WINDOW, frequency));
                      
            /* Configure reference level to desired value */
            CHECKERR (rsspecan_ConfigureReferenceLevel( instrSession, WINDOW, level));
            
            /* Set instrument to single sweep mode - continous mode is done
             * programmatically because this is the only way how to
             * synchronize measurement using *OPC
             */
            CHECKERR (rsspecan_ConfigureSignalStatisticSweep(instrSession, VI_FALSE));
                
            /* Configure trigger mode (external/free run) */
            CHECKERR (rsspecan_ConfigureTriggerSource (instrSession, 1, trigger_mode));

            SetWaitCursor(0);
            
            /* enable measurements */
            gRun = 1;

            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with error message.
/*           In case Device Clear is requested, it is necessary to set the
/*           argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{
    
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (instrSession && deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
       
    if (rsspecan_status < 0)
    {

        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsspecan_error_message (instrSession, rsspecan_status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", rsspecan_status, error_message);
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        
        MessagePopup ("Error", error_buffer);
        
        /* clean status after error handling */
        rsspecan_status = 0;        
    }
    
    /* close instrument session */
    if(instrSession)
    {
        rsspecan_close (instrSession);
        instrSession = VI_NULL;
    }
    
    /* no measurements */
    gRun  = 0;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
}


void InsertRow (ViInt32 row, ViString name, ViReal64 result, ViString units)
{
    ViInt32 rows;
    Point   cell;
    
    GetNumTableRows (panelHandle, PANEL_RESULTS_TABLE, &rows);
    cell.y = row;
    
    if ((row - 1) < rows)
    {
        cell.x = 2;
        SetTableCellVal (panelHandle, PANEL_RESULTS_TABLE, cell, result);
    }
    else
    {
        InsertTableRows (panelHandle, PANEL_RESULTS_TABLE, row, 1,
                         VAL_USE_MASTER_CELL_TYPE);
        cell.x = 1;
        SetTableCellVal (panelHandle, PANEL_RESULTS_TABLE, cell, name);
        cell.x = 2;
        SetTableCellVal (panelHandle, PANEL_RESULTS_TABLE, cell, result);
        cell.x = 3;
        SetTableCellVal (panelHandle, PANEL_RESULTS_TABLE, cell, units);
    }
    
}

/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons (ViBoolean enable)
{
    SetCtrlAttribute (panelHandle, PANEL_FREQ_NUMERIC,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_LEVEL_NUMERIC, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_TRIGGER_RING,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_SWEEP_RING,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAY_CHECKBOX,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_START_BUTTON,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESOURCE_STRING,   ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET_CHECKBOX,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP_BUTTON,   ATTR_DIMMED, enable);
}
